@@ -1,13 +1,28 @@
-"""Python library allowing interaction with the Cleverbot API."""
+"""An unofficial library to access the Cleverbot API."""
+from __future__ import (absolute_import, division, print_function,
+                        unicode_literals)
+from builtins import str  # pylint: disable=redefined-builtin
+from builtins import object  # pylint: disable=redefined-builtin
+
 import hashlib
-import urllib
-import urllib2
+import requests
+from requests.compat import urlencode
+from future.backports.html import parser
+
+# Only use the instance method `unescape` of entity_parser. (I wish it was a
+# static method or public function; it never uses `self` anyway)
+entity_parser = parser.HTMLParser()
 
 
-class Cleverbot:
-    """
-    Wrapper over the Cleverbot API.
+class Cleverbot(object):
+    """Handles a conversation with Cleverbot.
 
+    Example usage:
+
+       >>> from cleverbot import Cleverbot
+       >>> cb = Cleverbot()
+       >>> cb.ask("Hi. How are you?")
+       "I'm good, thanks. How are you?"
     """
     HOST = "www.cleverbot.com"
     PROTOCOL = "http://"
@@ -56,33 +71,30 @@ class Cleverbot:
 
         # the log of our conversation with Cleverbot
         self.conversation = []
-        self.resp = str()
+
+        # get the main page to get a cookie (see bug #13)
+        self.session = requests.Session()
+        self.session.get(Cleverbot.PROTOCOL + Cleverbot.HOST)
 
     def ask(self, question):
         """Asks Cleverbot a question.
-        
+
         Maintains message history.
 
-        Args:
-            q (str): The question to ask
-
-        Returns:
-            Cleverbot's answer
+        :param question: The question to ask
+        :return Cleverbot's answer
         """
+
         # Set the current question
         self.data['stimulus'] = question
 
         # Connect to Cleverbot's API and remember the response
-        try:
-            self.resp = self._send()
-        except urllib2.HTTPError:
-            # request failed. returning empty string
-            return str()
+        resp = self._send()
 
         # Add the current question to the conversation log
         self.conversation.append(question)
 
-        parsed = self._parse()
+        parsed = self._parse(resp.text)
 
         # Set data as appropriate
         if self.data['sessionid'] != '':
@@ -94,13 +106,16 @@ class Cleverbot:
         return parsed['answer']
 
     def _send(self):
-        """POST the user's question and all required information to the 
+        """POST the user's question and all required information to the
         Cleverbot API
 
         Cleverbot tries to prevent unauthorized access to its API by
-        obfuscating how it generates the 'icognocheck' token, so we have
-        to URLencode the data twice: once to generate the token, and
-        twice to add the token to the data we're sending to Cleverbot.
+        obfuscating how it generates the 'icognocheck' token. The token is
+        currently the md5 checksum of the 10th through 36th characters of the
+        encoded data. This may change in the future.
+
+        TODO: Order is not guaranteed when urlencoding dicts. This hasn't been
+        a problem yet, but let's look into ordered dicts or tuples instead.
         """
         # Set data as appropriate
         if self.conversation:
@@ -112,31 +127,39 @@ class Cleverbot:
                     break
 
         # Generate the token
-        enc_data = urllib.urlencode(self.data)
+        enc_data = urlencode(self.data)
         digest_txt = enc_data[9:35]
-        token = hashlib.md5(digest_txt).hexdigest()
+        token = hashlib.md5(digest_txt.encode('utf-8')).hexdigest()
         self.data['icognocheck'] = token
 
-        # Add the token to the data
-        enc_data = urllib.urlencode(self.data)
-        req = urllib2.Request(self.API_URL, enc_data, self.headers)
+        # POST the data to Cleverbot's API and return
+        return self.session.post(Cleverbot.API_URL,
+                                 data=self.data,
+                                 headers=Cleverbot.headers)
 
-        # POST the data to Cleverbot's API
-        conn = urllib2.urlopen(req)
-        resp = conn.read()
-
-        # Return Cleverbot's response
-        return resp
-
-    def _parse(self):
+    @staticmethod
+    def _parse(resp_text):
         """Parses Cleverbot's response"""
-        parsed = [
-            item.split('\r') for item in self.resp.split('\r\r\r\r\r\r')[:-1]
-        ]
+        resp_text = entity_parser.unescape(resp_text)
 
-        return {
+        parsed = [
+            item.split('\r') for item in resp_text.split('\r\r\r\r\r\r')[:-1]
+            ]
+
+        if parsed[0][1] == 'DENIED':
+            raise CleverbotAPIError()
+
+        parsed_dict = {
             'answer': parsed[0][0],
             'conversation_id': parsed[0][1],
             'conversation_log_id': parsed[0][2],
-            'unknown': parsed[1][-1]
         }
+        try:
+            parsed_dict['unknown'] = parsed[1][-1]
+        except IndexError:
+            parsed_dict['unknown'] = None
+        return parsed_dict
+
+
+class CleverbotAPIError(Exception):
+    """Cleverbot returned an error (it probably recognized us as a bot)"""
